@@ -15,23 +15,23 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/crypt/hash_sha2_512.h>
+#include "hash_sha2_512_256.h"
 
 namespace mobius
 {
 namespace crypt
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-// initialization constants (FIPS.180-4 - section 5.3.5)
+// initialization constants (FIPS.180-4 - section 5.3.6.2)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static constexpr std::uint64_t H0_VALUE = 0x6a09e667f3bcc908;
-static constexpr std::uint64_t H1_VALUE = 0xbb67ae8584caa73b;
-static constexpr std::uint64_t H2_VALUE = 0x3c6ef372fe94f82b;
-static constexpr std::uint64_t H3_VALUE = 0xa54ff53a5f1d36f1;
-static constexpr std::uint64_t H4_VALUE = 0x510e527fade682d1;
-static constexpr std::uint64_t H5_VALUE = 0x9b05688c2b3e6c1f;
-static constexpr std::uint64_t H6_VALUE = 0x1f83d9abfb41bd6b;
-static constexpr std::uint64_t H7_VALUE = 0x5be0cd19137e2179;
+static constexpr std::uint64_t H0_VALUE = 0x22312194fc2bf72c;
+static constexpr std::uint64_t H1_VALUE = 0x9f555fa3c84c64c2;
+static constexpr std::uint64_t H2_VALUE = 0x2393b86b6f53b151;
+static constexpr std::uint64_t H3_VALUE = 0x963877195940eabd;
+static constexpr std::uint64_t H4_VALUE = 0x96283ee2a88effe3;
+static constexpr std::uint64_t H5_VALUE = 0xbe5e1e2553863992;
+static constexpr std::uint64_t H6_VALUE = 0x2b0199fc2c85b8aa;
+static constexpr std::uint64_t H7_VALUE = 0x0eb72ddc81c52ca2;
 
 //! \brief block size = 128 bytes (1024 bits) - FIPS.180-4 - section 1
 static constexpr int BLOCK_SIZE = 128;
@@ -51,7 +51,7 @@ rrot (std::uint64_t v, std::uint64_t n) noexcept
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief default constructor
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-hash_sha2_512::hash_sha2_512 () noexcept
+hash_sha2_512_256::hash_sha2_512_256 () noexcept
   : hash_block (BLOCK_SIZE),
     a_ (H0_VALUE),
     b_ (H1_VALUE),
@@ -69,7 +69,7 @@ hash_sha2_512::hash_sha2_512 () noexcept
 //! \brief reset hash value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-hash_sha2_512::_reset () noexcept
+hash_sha2_512_256::_reset () noexcept
 {
   hash_block::reset ();
   a_ = H0_VALUE;
@@ -89,7 +89,7 @@ hash_sha2_512::_reset () noexcept
 //! \see FIPS.180-4 - section 6.2
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-hash_sha2_512::_update_block (const mobius::bytearray& data) noexcept
+hash_sha2_512_256::_update_block (const mobius::bytearray& data) noexcept
 {
   std::uint64_t a = a_;
   std::uint64_t b = b_;
@@ -302,323 +302,883 @@ hash_sha2_512::_update_block (const mobius::bytearray& data) noexcept
   // unrolled loop 0..79
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x428a2f98d728ae22 + W[0];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x7137449123ef65cd + W[1];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xb5c0fbcfec4d3b2f + W[2];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xe9b5dba58189dbbc + W[3];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x3956c25bf348b538 + W[4];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x59f111f1b605d019 + W[5];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x923f82a4af194f9b + W[6];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xab1c5ed5da6d8118 + W[7];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xd807aa98a3030242 + W[8];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x12835b0145706fbe + W[9];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x243185be4ee4b28c + W[10];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x550c7dc3d5ffb4e2 + W[11];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x72be5d74f27b896f + W[12];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x80deb1fe3b1696b1 + W[13];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x9bdc06a725c71235 + W[14];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xc19bf174cf692694 + W[15];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xe49b69c19ef14ad2 + W[16];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xefbe4786384f25e3 + W[17];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xfc19dc68b8cd5b5 + W[18];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x240ca1cc77ac9c65 + W[19];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x2de92c6f592b0275 + W[20];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x4a7484aa6ea6e483 + W[21];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x5cb0a9dcbd41fbd4 + W[22];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x76f988da831153b5 + W[23];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x983e5152ee66dfab + W[24];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xa831c66d2db43210 + W[25];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xb00327c898fb213f + W[26];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xbf597fc7beef0ee4 + W[27];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xc6e00bf33da88fc2 + W[28];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xd5a79147930aa725 + W[29];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x6ca6351e003826f + W[30];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x142929670a0e6e70 + W[31];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x27b70a8546d22ffc + W[32];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x2e1b21385c26c926 + W[33];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x4d2c6dfc5ac42aed + W[34];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x53380d139d95b3df + W[35];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x650a73548baf63de + W[36];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x766a0abb3c77b2a8 + W[37];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x81c2c92e47edaee6 + W[38];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x92722c851482353b + W[39];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xa2bfe8a14cf10364 + W[40];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xa81a664bbc423001 + W[41];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xc24b8b70d0f89791 + W[42];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xc76c51a30654be30 + W[43];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xd192e819d6ef5218 + W[44];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xd69906245565a910 + W[45];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xf40e35855771202a + W[46];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x106aa07032bbd1b8 + W[47];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x19a4c116b8d2d0c8 + W[48];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x1e376c085141ab53 + W[49];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x2748774cdf8eeb99 + W[50];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x34b0bcb5e19b48a8 + W[51];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x391c0cb3c5c95a63 + W[52];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x4ed8aa4ae3418acb + W[53];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x5b9cca4f7763e373 + W[54];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x682e6ff3d6b2b8a3 + W[55];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x748f82ee5defb2fc + W[56];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x78a5636f43172f60 + W[57];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x84c87814a1f0ab72 + W[58];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x8cc702081a6439ec + W[59];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x90befffa23631e28 + W[60];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xa4506cebde82bde9 + W[61];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xbef9a3f7b2c67915 + W[62];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xc67178f2e372532b + W[63];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xca273eceea26619c + W[64];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xd186b8c721c0c207 + W[65];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xeada7dd6cde0eb1e + W[66];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xf57d4f7fee6ed178 + W[67];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x6f067aa72176fba + W[68];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0xa637dc5a2c898a6 + W[69];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x113f9804bef90dae + W[70];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x1b710b35131c471b + W[71];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x28db77f523047d84 + W[72];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x32caab7b40c72493 + W[73];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x3c9ebe0a15c9bebc + W[74];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x431d67c49c100d4c + W[75];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x4cc5d4becb3e42b6 + W[76];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x597f299cfc657e2a + W[77];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x5fcb6fab3ad6faec + W[78];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   temp1 = h + (rrot (e, 14) ^ rrot (e, 18) ^ rrot (e, 41)) + ((e & f) ^ ((~e) & g)) + 0x6c44198c4a475817 + W[79];
   temp2 = (rrot (a, 28) ^ rrot (a, 34) ^ rrot (a, 39)) + ((a & b) ^ (a & c) ^ (b & c));
-  h = g; g = f; f = e; e = d + temp1; d = c; c = b; b = a; a = temp1 + temp2;
+  h = g;
+  g = f;
+  f = e;
+  e = d + temp1;
+  d = c;
+  c = b;
+  b = a;
+  a = temp1 + temp2;
 
   // update keys
   a_ += a;
@@ -642,7 +1202,7 @@ hash_sha2_512::_update_block (const mobius::bytearray& data) noexcept
 // representing the message size in bits (FIPS.180-4 - section 5.1.2)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 const mobius::bytearray
-hash_sha2_512::_evaluate (const mobius::bytearray& data)
+hash_sha2_512_256::_evaluate (const mobius::bytearray& data)
 {
   // represent message size in bits as a bytearray
   std::uint64_t bits = ((size_ + data.size ()) << 3);
@@ -727,38 +1287,6 @@ hash_sha2_512::_evaluate (const mobius::bytearray& data)
     std::uint8_t (d_ >> 16),
     std::uint8_t (d_ >> 8),
     std::uint8_t (d_),
-    std::uint8_t (e_ >> 56),
-    std::uint8_t (e_ >> 48),
-    std::uint8_t (e_ >> 40),
-    std::uint8_t (e_ >> 32),
-    std::uint8_t (e_ >> 24),
-    std::uint8_t (e_ >> 16),
-    std::uint8_t (e_ >> 8),
-    std::uint8_t (e_),
-    std::uint8_t (f_ >> 56),
-    std::uint8_t (f_ >> 48),
-    std::uint8_t (f_ >> 40),
-    std::uint8_t (f_ >> 32),
-    std::uint8_t (f_ >> 24),
-    std::uint8_t (f_ >> 16),
-    std::uint8_t (f_ >> 8),
-    std::uint8_t (f_),
-    std::uint8_t (g_ >> 56),
-    std::uint8_t (g_ >> 48),
-    std::uint8_t (g_ >> 40),
-    std::uint8_t (g_ >> 32),
-    std::uint8_t (g_ >> 24),
-    std::uint8_t (g_ >> 16),
-    std::uint8_t (g_ >> 8),
-    std::uint8_t (g_),
-    std::uint8_t (h_ >> 56),
-    std::uint8_t (h_ >> 48),
-    std::uint8_t (h_ >> 40),
-    std::uint8_t (h_ >> 32),
-    std::uint8_t (h_ >> 24),
-    std::uint8_t (h_ >> 16),
-    std::uint8_t (h_ >> 8),
-    std::uint8_t (h_),
   };
 }
 
